package example.jdbc_rmi.server;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;

import example.jdbc_rmi.api.LoginResult;
import example.jdbc_rmi.api.LoginService;
import external.BCrypt;

public class LoginServiceImpl implements LoginService {

    // Database connection options
    private static final String HOST = "localhost";
    private static final int PORT = 1527;
    private static final String NAME = "userdb";

    private final Connection connection;

    public LoginServiceImpl() throws RemoteException {
        // Assemble the 'connection string'.
        // The 'create=true' option means the Derby server will create the
        // database, if it does not exist yet.
        String connectionString = "jdbc:derby://" + HOST + ":" + PORT + "/" + NAME + ";create=true";

        Connection c = null;

        try {
            // Connect to the database
            c = DriverManager.getConnection(connectionString);
        } catch (SQLException ex) {
            Logger.getLogger(LoginServiceImpl.class.getName()).log(Level.SEVERE, null, ex);
            throw new RemoteException("Database connection error.", ex);
        }

        // Save the new connection
        connection = c;

        try {
            // Start a transaction
            connection.setAutoCommit(false);

            // Check that our table exists
            DatabaseMetaData metaData = connection.getMetaData();

            // Filter only by table name (note the upper case), and type
            ResultSet tables = metaData.getTables(null, null, "SERVICE_USERS", new String[] { "TABLE" });

            // Check that the result set is empty
            if(!tables.next()) {
                // Create a statement that will create the table
                try (Statement stmt = connection.createStatement()) {
                    stmt.execute(
                        "CREATE TABLE service_users (" +
                            "name VARCHAR(255)," +
                            "password VARCHAR(255)," +
                            "level INT," +
                            "PRIMARY KEY(name)" +
                        ")"
                    );
                }

                // Fill the table with users (passwords are encrypted with bcrypt)
                try (Statement stmt = connection.createStatement()) {
                    stmt.execute("INSERT INTO service_users VALUES('A', '$2a$10$ZqrV98f/B9V5.P5I7SxoSONXvi8XqPKk0ad4vJYW/TAfhDg9CmG76', 1)");
                    stmt.execute("INSERT INTO service_users VALUES('B', '$2a$10$3lL.ehd7WjPcE37592JqMuE9Pyun1hZfhB.D7kmEmmPBqRY6VXcr.', 2)");
                    stmt.execute("INSERT INTO service_users VALUES('C', '$2a$10$.rdCup4V5mwhHWNAH8ISVurbgnOobFOeo7/zTAFknIFnM5dhF5cae', 3)");
                }
            }

            tables.close();

            // Commit the transaction
            connection.commit();
        } catch (SQLException ex) {
            try {
                // Invalidate transaction
                connection.rollback();
            } catch (SQLException e) {}

            Logger.getLogger(LoginServiceImpl.class.getName()).log(Level.SEVERE, null, ex);
            throw new RemoteException("Database error.", ex);
        }
    }

    @Override
    public LoginResult login(String userName, String password) throws RemoteException {
        // Create a prepared statement and select the user by name
        try (PreparedStatement pstmt
                = connection.prepareStatement("SELECT * FROM service_users WHERE name = ?")) {
            // Fill in the parameter
            pstmt.setString(1, userName);

            try (ResultSet result = pstmt.executeQuery()) {
                // Check whether a result is found
                if (result.next()) {
                    // Get the stored password hash
                    String hash = result.getString("password");

                    // Check the given password with the original hash
                    System.out.println(BCrypt.hashpw(password, BCrypt.gensalt()));

                    if (BCrypt.checkpw(password, hash)) {
                        int level = result.getInt("level");

                        // Send back the user level
                        return new LoginResult(
                            LoginResult.Outcome.SUCCESSFUL,
                            "Successfully logged in with user level " + level + "!"
                        );
                    }
                }
            }
        } catch (SQLException ex) {
            Logger.getLogger(LoginServiceImpl.class.getName()).log(Level.SEVERE, null, ex);

            // Indicate an error for the client
            return new LoginResult(LoginResult.Outcome.INTERNAL_ERROR, "An unexpected error occured. Please contact an administrator.");
        }

        // Indicate that the given credentials are invalid
        return new LoginResult(LoginResult.Outcome.INVALID_CREDENTIALS, "Invalid credentials!");
    }

}
