package main.jpa;

import java.util.List;
import java.util.function.Consumer;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

import entities.Inventory;

public class Main {
    public static void main(String[] args) {
        withDbTransaction("testpu", em -> {
            deleteRecords(em);

            insertRecords(em);
            selectRecords(em);

            // Clear the entity cache to force re-select next time
            em.clear();

            updateRecords(em);
            selectRecords(em);
        });
    }

    private static void withDbTransaction(String puName, Consumer<EntityManager> transaction) {
        // Get an entity manager factory by persistence unit name
        // The database table will be automatically created by eclipselink
        EntityManagerFactory emf = Persistence.createEntityManagerFactory(puName);

        // Create an entity manager
        EntityManager em = emf.createEntityManager();

        // Begin transaction
        em.getTransaction().begin();

        // The caller has specified what to do, run those steps now...
        transaction.accept(em);

        // Commit the transaction and close the entity manager
        em.getTransaction().commit();
        em.close();
    }

    private static void deleteRecords(EntityManager em) {
        // Bulk delete with criteria API
        Query query = em.createQuery("DELETE FROM Inventory");
        int removedItems = query.executeUpdate();

        if (removedItems > 0) {
            System.out.println(removedItems + " items removed.");
        }
    }

    private static void insertRecords(EntityManager em) {
        for(int i = 0; i < 5; ++i) {
            insertRecord(em, i);
        }
    }

	private static void insertRecord(EntityManager em, int i) {
		Inventory inv = makeInventory(em, i);
		em.persist(inv);
	}

	private static Inventory makeInventory(EntityManager em, int i) {
		Inventory inv = new Inventory();
		inv.setName("Item #" + i);
		inv.setQuantity(i + 2);
		inv.setPrice(i * 1.41f);
		return inv;
	}

    private static void selectRecords(EntityManager em) {
        // Select with JPQL and print out entities
        TypedQuery<Inventory> q = em.createQuery("SELECT i FROM Inventory AS i ORDER BY i.name", Inventory.class);
        List<Inventory> items = q.getResultList();

        System.out.println();

        for(Inventory i : items) {
            System.out.println(i);
        }

        System.out.println();
    }

    private static void updateRecords(EntityManager em) {
        System.out.println("Updating items with price 0 to 100...");

        // Update with criteria API
        Query query =
        		em.createQuery("UPDATE Inventory AS i SET i.price = :originalPrice WHERE i.price = :newPrice")
        			.setParameter("originalPrice", 100.0)
        			.setParameter("newPrice", 0.0);
        int updatedItems = query.executeUpdate();

        if (updatedItems > 0) {
            System.out.println(updatedItems + " items updated.");
        }
    }
}
