package example.tx;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJBContext;
import javax.ejb.EJBException;
import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import example.tx.entities.Message;
import example.tx.exceptions.FatalMessageException;
import example.tx.exceptions.MessageException;
import example.tx.interceptors.InvocationLogger;

@Stateful
// Let the bean manage the transactions.
@TransactionManagement(TransactionManagementType.BEAN)
// Log all invocations
@Interceptors({ InvocationLogger.class })
public class CustomTxStatefulMessagesBean implements Messages {

	@PersistenceContext
	private EntityManager em;

	private boolean hasTx = false;

	@Resource
	private EJBContext ctx;

	private UserTransaction tx;

	@PostConstruct
	private void init() {
		tx = ctx.getUserTransaction();
	}

	// Alternative:
	// @Resource private UserTransaction tx;

	@Override
	public void add(String text, ErrorType errorType) throws MessageException {
		// Simulate a REQUIRED transaction
		if (!hasTx) {
			try {
				tx.begin();
				System.out.println("Transaction begin");
				hasTx = true;
			} catch (IllegalStateException | NotSupportedException
					| SystemException e) {
				throw new EJBException(e);
			}
		}

		// Try to persist the message
		em.persist(new Message(text));

		// Throw the specified exception
		switch (errorType) {
		case Runtime:
			throw new RuntimeException("Runtime exception thrown.");
		case Sys:
			throw new EJBException("System exception thrown.");
		case AppNonRollback:
			throw new MessageException("Non-rollback app exception thrown.");
		case AppRollback:
			throw new FatalMessageException("Rollback app exception thrown.");
		default:
			// Intentionally left blank, do nothing
		}
	}

	@Override
	@Remove
	public List<String> getMessages() {
		// Simulate a MANDATORY transaction
		if (!hasTx) {
			throw new EJBException("Please start a transaction.");
		}

		List<String> messages = em.createQuery("SELECT m.text FROM Message m",
				String.class).getResultList();

		try {
			if (Status.STATUS_ACTIVE == tx.getStatus()) {
				tx.commit();
				System.out.println("Transaction commit");
			} else {
				System.out.println("Transaction commit failed: " + tx.getStatus());
			}
			hasTx = false;
		} catch (SecurityException | IllegalStateException | RollbackException
				| HeuristicMixedException | HeuristicRollbackException
				| SystemException e) {
			throw new EJBException(e);
		}

		return messages;
	}
}
